@@ -1,6 +1,6 @@
 /*************************************************************************************************
  * Implementation of Depot
- *                                                      Copyright (C) 2000-2004 Mikio Hirabayashi
+ *                                                      Copyright (C) 2000-2005 Mikio Hirabayashi
  * This file is part of QDBM, Quick Database Manager.
  * QDBM is free software; you can redistribute it and/or modify it under the terms of the GNU
  * Lesser General Public License as published by the Free Software Foundation; either version
@@ -125,7 +125,7 @@ const char *dperrmsg(int ecode){
 /* Get a database handle. */
 DEPOT *dpopen(const char *name, int omode, int bnum){
   int mode, fd, inode, mtime, fsiz, rnum, msiz;
-  char hbuf[DP_HEADSIZ], *map;
+  char hbuf[DP_HEADSIZ], *map, c;
   struct stat sbuf;
   DEPOT *depot;
   assert(name);
@@ -173,19 +173,30 @@ DEPOT *dpopen(const char *name, int omode, int bnum){
     memcpy(hbuf + DP_RNUMOFF, &rnum, sizeof(int));
     fsiz = DP_HEADSIZ + bnum * sizeof(int);
     memcpy(hbuf + DP_FSIZOFF, &fsiz, sizeof(int));
-    if(!(map = malloc(bnum * sizeof(int)))){
+    if(!dpseekwrite(fd, 0, hbuf, DP_HEADSIZ)){
       close(fd);
-      dpecodeset(DP_EALLOC, __FILE__, __LINE__);
       return NULL;
     }
-    memset(map, 0, bnum * sizeof(int));
-    if(!dpseekwrite(fd, 0, hbuf, DP_HEADSIZ) ||
-       !dpseekwrite(fd, DP_HEADSIZ, map, bnum * sizeof(int))){
+    if(omode & DP_OSPARSE){
+      c = 0;
+      if(!dpseekwrite(fd, fsiz - 1, &c, 1)){
+        close(fd);
+        return NULL;
+      }
+    } else {
+      if(!(map = malloc(bnum * sizeof(int)))){
+        close(fd);
+        dpecodeset(DP_EALLOC, __FILE__, __LINE__);
+        return NULL;
+      }
+      memset(map, 0, bnum * sizeof(int));
+      if(!dpseekwrite(fd, DP_HEADSIZ, map, bnum * sizeof(int))){
+        free(map);
+        close(fd);
+        return NULL;
+      }
       free(map);
-      close(fd);
-      return NULL;
     }
-    free(map);
   }
   if(!dpseekread(fd, 0, hbuf, DP_HEADSIZ)){
     close(fd);
@@ -1679,8 +1690,8 @@ static int dprecrewrite(DEPOT *depot, int off, int rsiz, const char *kbuf, int k
    The return value is the offset of the record, or, -1 on failure. */
 static int dprecappend(DEPOT *depot, const char *kbuf, int ksiz, const char *vbuf, int vsiz,
                        int hash, int left, int right){
-  char ebuf[DP_WRTBUFSIZ];
-  int head[DP_RHNUM], asiz, psiz, off, hoff, koff, voff, c;
+  char ebuf[DP_WRTBUFSIZ], *hbuf;
+  int head[DP_RHNUM], asiz, psiz, off;
   assert(depot && kbuf && ksiz >= 0 && vbuf && vsiz >= 0);
   psiz = dppadsize(depot, vsiz);
   head[DP_RHIFLAGS] = 0;
@@ -1699,14 +1710,19 @@ static int dprecappend(DEPOT *depot, const char *kbuf, int ksiz, const char *vbu
     memset(ebuf + sizeof(head) + ksiz + vsiz, 0, psiz);
     if(!dpseekwrite(depot->fd, off, ebuf, asiz)) return -1;
   } else {
-    hoff = off;
-    koff = hoff + sizeof(head);
-    voff = koff + ksiz;
-    c = 0;
-    if(!dpseekwrite(depot->fd, hoff, head, sizeof(head)) ||
-       !dpseekwrite(depot->fd, koff, kbuf, ksiz) ||
-       !dpseekwrite(depot->fd, voff, vbuf, vsiz) ||
-       (psiz > 0 && !dpseekwrite(depot->fd, voff + vsiz + psiz - 1, &c, 1))) return -1;
+    if(!(hbuf = malloc(asiz))){
+      dpecodeset(DP_EALLOC, __FILE__, __LINE__);
+      return -1;
+    }
+    memcpy(hbuf, head, sizeof(head));
+    memcpy(hbuf + sizeof(head), kbuf, ksiz);
+    memcpy(hbuf + sizeof(head) + ksiz, vbuf, vsiz);
+    memset(hbuf + sizeof(head) + ksiz + vsiz, 0, psiz);
+    if(!dpseekwrite(depot->fd, off, hbuf, asiz)){
+      free(hbuf);
+      return -1;
+    }
+    free(hbuf);
   }
   depot->fsiz += asiz;
   return off;
