@@ -1,6 +1,6 @@
 /*************************************************************************************************
  * Raving Bulletin Board System
- *                                                      Copyright (C) 2003-2004 Mikio Hirabayashi
+ *                                                      Copyright (C) 2003-2005 Mikio Hirabayashi
  * This file is part of RBBS, a web-based bulletin board system.
  * RBBS is free software; you can redistribute it and/or modify it under the terms of the GNU
  * General Public License as published by the Free Software Foundation; either version 2 of the
@@ -24,7 +24,7 @@
 #define RXMLTYPE    "application/xml"    /* content-type of raw XML */
 #define ENCODING    "UTF-8"              /* encoding of the page */
 #define PETITBNUM   31                   /* bucket number of a petit map */
-#define RDATAMAX    1048576              /* max size of data to read */
+#define RDATAMAX    67108864             /* max size of data to read */
 #define PAGENAME    "rbbs.html"          /* saving filename of the page */
 #define FEEDNAME    "rbbs.xml"           /* saving filename of the feed */
 #define FEEDCDATE   "cdate"              /* ID for feed ordered by creation date */
@@ -33,9 +33,11 @@
 #define DUMPDBCMD   "dumpdb"             /* command name to dump all articles */
 #define DELEDBCMD   "deledb"             /* command name to delete the database */
 #define CRPREFIX    "crypt:"             /* prefix for password of criptgraph */
+#define MEMOATTR    "memo"               /* name of the attribute for memorandum */
 #define TOPICCHAR   '!'                  /* beginning character for topic */
 #define ITEMCHAR    '*'                  /* beginning character for list item */
 #define ASISCHAR    '|'                  /* beginning character for asis text */
+#define BREAKSTR    "---"                /* string for breaking line */
 #define SINKPREFIX  SINKSTR ":"          /* prefix for sinking responses */
 
 enum {                                   /* enumeration for actions */
@@ -50,6 +52,11 @@ enum {                                   /* enumeration for actions */
   ACTPOSTRES,                            /* post a response */
   ACTDUMPDB,                             /* dump all articles */
   ACTDELEDB,                             /* delete the database */
+  ACTLISTFILE,                           /* show the list of files */
+  ACTSAVEFILE,                           /* upload and save a file */
+  ACTDELEFILE,                           /* delete a file */
+  ACTSHOWMEMO,                           /* show the memorandum */
+  ACTPOSTMEMO,                           /* post the memorandum */
   ACTHELP                                /* show the help page */
 };
 
@@ -81,6 +88,8 @@ const char *g_artflags = NULL;           /* default flags for a new article */
 const char *g_resflags = NULL;           /* default flags for a new response */
 const char *g_languages = NULL;          /* list of supported languages */
 const char *g_categories = NULL;         /* list of supported categories */
+const char *g_defsubject = NULL;         /* subject by default */
+const char *g_defauthor = NULL;          /* author by default */
 int g_sort = 0;                          /* default sorting order of articles */
 int g_listnum = 0;                       /* number of shown items in the subject list */
 int g_shownum = 0;                       /* number of shown articles in the page */
@@ -90,6 +99,8 @@ int g_resnum = 0;                        /* number of shown responses for an art
 const char *g_homelabel = NULL;          /* label to go to the home page */
 const char *g_helplabel = NULL;          /* label to show the help page */
 const char *g_postlabel = NULL;          /* label to post a new article */
+const char *g_filelabel = NULL;          /* label to manage files */
+const char *g_memolabel = NULL;          /* label to show the memo page */
 const char *g_feedlabel = NULL;          /* label to get the feed the page or an article */
 const char *g_reloadlabel = NULL;        /* label to reload the page */
 const char *g_searchlabel = NULL;        /* label to search articles */
@@ -118,6 +129,12 @@ const char *g_submitlabel = NULL;        /* label for general submit button */
 const char *g_resetlabel = NULL;         /* label for general reset button */
 const char *g_backlabel = NULL;          /* label for general back button */
 const char *g_anykindlabel = NULL;       /* label for selection of any kind */
+const char *g_pathlabel = NULL;          /* label for file path */
+const char *g_sizelabel = NULL;          /* label for file size */
+const char *g_typelabel = NULL;          /* label for file type */
+const char *g_actionlabel = NULL;        /* label for file actions */
+const char *g_getlabel = NULL;           /* label for file getting */
+const char *g_dellabel = NULL;           /* label for file deleting */
 
 
 /* global variables for parameters, cookies, and authentication */
@@ -138,6 +155,10 @@ const char *p_hash = NULL;               /* hash code of an article */
 int p_year = 1;                          /* creation year of articles to be shown */
 const char *p_cdate = NULL;              /* creation date of articles to be shown */
 const char *p_feed = NULL;               /* ID of an article for Atom feed */
+const char *p_file = NULL;               /* content of the uploaded file */
+int p_filelen = 0;                       /* length of the content of the uploaded file */
+const char *p_filename = NULL;           /* name of the uploaded file */
+const char *c_userpass = NULL;           /* userpass in the cookie */
 const char *c_author = NULL;             /* author in the cookie */
 const char *a_author = NULL;             /* author via basic authentication */
 
@@ -148,7 +169,6 @@ const char *g_curdate = NULL;            /* current date string */
 int g_curyear = 1;                       /* current year */
 int g_curmonth = 1;                      /* current month */
 int g_curday = 1;                        /* current day */
-int g_rmpost = FALSE;                    /* whether the request method is POST */
 int g_cannext = FALSE;                   /* whether the next page can be shown */
 const char *g_rtryres = "";              /* text for re-try to post a response */
 int g_tabidx = 0;                        /* counter for tab indexes */
@@ -164,10 +184,13 @@ void senderror(const char *msg);
 void sendnotfound(void);
 void senddump(void);
 void sendfeed(void);
+void sendasis(const char *id);
+const char *getmimetype(const char *name);
 void sendhtml(void);
 int canadmin(void);
 int canwrite(void);
 int canartadmin(void);
+int canfileadmin(void);
 int authorize(void);
 void shownote(const char *msg);
 void showpagenavi(void);
@@ -189,6 +212,8 @@ void mypostarticle(void);
 void mydeletearticle(void);
 int mypostresponse(void);
 void deletedb(void);
+void mylistfiles(void);
+void showmemo(void);
 void showhelp(void);
 
 
@@ -198,7 +223,7 @@ int main(int argc, char **argv){
   CBMAP *params, *cooks;
   const char *tmp;
   char *buf;
-  int i;
+  int i, ispost;
   /* set configurations */
   signal(SIGINT, SIG_IGN);
   signal(SIGQUIT, SIG_IGN);
@@ -250,6 +275,10 @@ int main(int argc, char **argv){
       g_languages = skiplabel(tmp);
     } else if(cbstrfwimatch(tmp, "categories:")){
       g_categories = skiplabel(tmp);
+    } else if(cbstrfwimatch(tmp, "defauthor:")){
+      g_defauthor = skiplabel(tmp);
+    } else if(cbstrfwimatch(tmp, "defsubject:")){
+      g_defsubject = skiplabel(tmp);
     } else if(cbstrfwimatch(tmp, "sort:")){
       tmp = skiplabel(tmp);
       if(!strcmp(tmp, "cda")){
@@ -277,6 +306,10 @@ int main(int argc, char **argv){
       g_helplabel = skiplabel(tmp);
     } else if(cbstrfwimatch(tmp, "postlabel:")){
       g_postlabel = skiplabel(tmp);
+    } else if(cbstrfwimatch(tmp, "filelabel:")){
+      g_filelabel = skiplabel(tmp);
+    } else if(cbstrfwimatch(tmp, "memolabel:")){
+      g_memolabel = skiplabel(tmp);
     } else if(cbstrfwimatch(tmp, "feedlabel:")){
       g_feedlabel = skiplabel(tmp);
     } else if(cbstrfwimatch(tmp, "reloadlabel:")){
@@ -333,6 +366,18 @@ int main(int argc, char **argv){
       g_backlabel = skiplabel(tmp);
     } else if(cbstrfwimatch(tmp, "anykindlabel:")){
       g_anykindlabel = skiplabel(tmp);
+    } else if(cbstrfwimatch(tmp, "pathlabel:")){
+      g_pathlabel = skiplabel(tmp);
+    } else if(cbstrfwimatch(tmp, "sizelabel:")){
+      g_sizelabel = skiplabel(tmp);
+    } else if(cbstrfwimatch(tmp, "typelabel:")){
+      g_typelabel = skiplabel(tmp);
+    } else if(cbstrfwimatch(tmp, "actionlabel:")){
+      g_actionlabel = skiplabel(tmp);
+    } else if(cbstrfwimatch(tmp, "getlabel:")){
+      g_getlabel = skiplabel(tmp);
+    } else if(cbstrfwimatch(tmp, "dellabel:")){
+      g_dellabel = skiplabel(tmp);
     }
   }
   if(!g_systemuri) senderror("systemuri is undefined.");
@@ -352,6 +397,8 @@ int main(int argc, char **argv){
   if(!g_resflags) senderror("resflags is undefined.");
   if(!g_languages) senderror("languages is undefined.");
   if(!g_categories) senderror("categories is undefined.");
+  if(!g_defsubject) senderror("defsubject is undefined.");
+  if(!g_defauthor) senderror("defauthor is undefined.");
   if(g_sort < SORTCDATEA || g_sort >  SORTMDATED) g_sort = SORTCDATEA;
   if(g_listnum < 0) g_listnum = 0;
   if(g_shownum < 0) g_shownum = 0;
@@ -359,6 +406,8 @@ int main(int argc, char **argv){
   if(!g_homelabel) senderror("homelabel is undefined.");
   if(!g_helplabel) senderror("helplabel is undefined.");
   if(!g_postlabel) senderror("postlabel is undefined.");
+  if(!g_filelabel) senderror("filelabel is undefined.");
+  if(!g_memolabel) senderror("memolabel is undefined.");
   if(!g_feedlabel) senderror("feedlabel is undefined.");
   if(!g_reloadlabel) senderror("reloadlabel is undefined.");
   if(!g_searchlabel) senderror("searchlabel is undefined.");
@@ -387,6 +436,12 @@ int main(int argc, char **argv){
   if(!g_resetlabel) senderror("resetlabel is undefined");
   if(!g_backlabel) senderror("backlabel is undefined");
   if(!g_anykindlabel) senderror("anykindlabel is undefined");
+  if(!g_pathlabel) senderror("pathlabel is undefined");
+  if(!g_sizelabel) senderror("sizelabel is undefined");
+  if(!g_typelabel) senderror("typelabel is undefined");
+  if(!g_actionlabel) senderror("actionlabel is undefined");
+  if(!g_getlabel) senderror("getlabel is undefined");
+  if(!g_dellabel) senderror("dellabel is undefined");
   /* read parameters */
   params = getparameters();
   cbglobalgc(params, (void (*)(void *))cbmapclose);
@@ -431,15 +486,25 @@ int main(int argc, char **argv){
   if(p_year < 1 || p_year > 9999) p_year = 1;
   if(!p_cdate) p_cdate = "";
   if(!p_feed) p_feed = "";
+  if(!(p_file = cbmapget(params, "file", -1, &p_filelen))){
+    p_file = "";
+    p_filelen = 0;
+  }
+  if(!(p_filename = cbmapget(params, "file-filename", -1, NULL))) p_filename = "";
+  if((tmp = cbmapget(params, "filename", -1, NULL)) != NULL && tmp[0] != '\0') p_filename = tmp;
   /* read cookies */
   cooks = getcookies();
   cbglobalgc(cooks, (void (*)(void *))cbmapclose);
+  if((tmp = cbmapget(cooks, "userpass", -1, NULL)) != NULL) c_userpass = tmp;
+  if(!c_userpass) c_userpass = "";
   if((tmp = cbmapget(cooks, "author", -1, NULL)) != NULL) c_author = tmp;
   if(!c_author) c_author = "";
   /* read authentication */
   a_author = "";
   if(g_httpauth && (tmp = getenv("REMOTE_USER")) != NULL) a_author = tmp;
   /* open the database */
+  ispost = FALSE;
+  if((tmp = getenv("REQUEST_METHOD")) && !strcmp(tmp, "POST")) ispost = TRUE;
   switch(p_action){
   case ACTVIEW:
   case ACTCAL:
@@ -451,14 +516,29 @@ int main(int argc, char **argv){
     break;
   case ACTPOSTART:
   case ACTDELEART:
-    g_rbbs = rbbsopen(g_dbname, g_rmpost);
+    g_rbbs = rbbsopen(g_dbname, ispost);
     break;
   case ACTPOSTRES:
-    g_rbbs = rbbsopen(g_dbname, g_rmpost && p_text != '\0');
+    g_rbbs = rbbsopen(g_dbname, ispost && p_text != '\0');
     break;
   case ACTDUMPDB:
     g_rbbs = rbbsopen(g_dbname, FALSE);
     break;
+  case ACTSHOWMEMO:
+    g_rbbs = rbbsopen(g_dbname, FALSE);
+    break;
+  case ACTPOSTMEMO:
+    g_rbbs = rbbsopen(g_dbname, TRUE);
+    break;
+  case ACTLISTFILE:
+    g_rbbs = rbbsopen(g_dbname, FALSE);
+    break;
+  case ACTSAVEFILE:
+    g_rbbs = rbbsopen(g_dbname, p_file && p_filelen > 0);
+    break;
+  case ACTDELEFILE:
+    g_rbbs = rbbsopen(g_dbname, p_id[0] != '\0');
+    break;
   default:
     break;
   }
@@ -467,17 +547,24 @@ int main(int argc, char **argv){
   buf = datecurrent();
   cbglobalgc(buf, free);
   g_curdate = buf;
-  g_curyear = atoi(g_curdate);
-  g_curmonth = atoi(g_curdate + 5);
-  g_curday = atoi(g_curdate + 8);
+  buf = dateforwww(g_curdate, TRUE);
+  g_curyear = atoi(buf);
+  g_curmonth = atoi(buf + 5);
+  g_curday = atoi(buf + 8);
+  free(buf);
   /* show page frame */
   if(p_action == ACTDUMPDB){
     senddump();
   } else if(p_feed[0] != '\0'){
     sendfeed();
+  } else if((tmp = getenv("PATH_INFO")) != NULL){
+    if(*tmp == '/') tmp++;
+    sendasis(tmp);
   } else {
     sendhtml();
   }
+  /* create the database if no */
+  if(!g_rbbs && (g_rbbs = rbbsopen(g_dbname, TRUE)) != NULL) rbbsclose(g_rbbs);
   return 0;
 }
 
@@ -495,44 +582,71 @@ const char *skiplabel(const char *str){
 
 /* get a map of the CGI parameters */
 CBMAP *getparameters(void){
-  CBMAP *map;
-  CBLIST *pairs;
-  char *rbuf, *buf, *key, *val, *dkey, *dval;
-  const char *tmp;
-  int i, len, c;
+  CBMAP *map, *attrs;
+  CBLIST *pairs, *parts;
+  const char *tmp, *body;
+  char *buf, *key, *val, *dkey, *dval, *wp, *bound, *fbuf, *aname;
+  int i, len, c, blen, flen;
   map = cbmapopenex(PETITBNUM);
-  rbuf = NULL;
   buf = NULL;
-  if((tmp = getenv("CONTENT_LENGTH")) != NULL && (len = atoi(tmp)) > 0 && len <= RDATAMAX){
-    rbuf = cbmalloc(len + 1);
+  len = 0;
+  if((tmp = getenv("REQUEST_METHOD")) != NULL && !strcmp(tmp, "POST") &&
+     (tmp = getenv("CONTENT_LENGTH")) != NULL && (len = atoi(tmp)) > 0){
+    if(len > RDATAMAX) len = RDATAMAX;
+    buf = cbmalloc(len + 1);
     for(i = 0; i < len && (c = getchar()) != EOF; i++){
-      rbuf[i] = c;
+      buf[i] = c;
     }
-    rbuf[i] = '\0';
-    if(i == len) buf = rbuf;
-    g_rmpost = TRUE;
-  } else {
-    buf = getenv("QUERY_STRING");
+    buf[i] = '\0';
+    if(i != len){
+      free(buf);
+      buf = NULL;
+    }
+  } else if((tmp = getenv("QUERY_STRING")) != NULL){
+    buf = cbmemdup(tmp, -1);
+    len = strlen(buf);
   }
-  if(buf != NULL){
-    buf = cbmemdup(buf, -1);
-    pairs = cbsplit(buf, -1, "&");
-    for(i = 0; i < cblistnum(pairs); i++){
-      key = cbmemdup(cblistval(pairs, i, NULL), -1);
-      if((val = strchr(key, '=')) != NULL){
-        *(val++) = '\0';
-        dkey = cburldecode(key, NULL);
-        dval = cburldecode(val, NULL);
-        cbmapput(map, dkey, -1, dval, -1, FALSE);
-        free(dval);
-        free(dkey);
+  if(buf && len > 0){
+    if((tmp = getenv("CONTENT_TYPE")) != NULL && cbstrfwmatch(tmp, "multipart/form-data") &&
+       (tmp = strstr(tmp, "boundary=")) != NULL){
+      tmp += 9;
+      bound = cbmemdup(tmp, -1);
+      if((wp = strchr(bound, ';')) != NULL) *wp = '\0';
+      parts = cbmimeparts(buf, len, bound);
+      for(i = 0; i < cblistnum(parts); i++){
+        body = cblistval(parts, i, &blen);
+        attrs = cbmapopenex(PETITBNUM);
+        fbuf = cbmimebreak(body, blen, attrs, &flen);
+        if((tmp = cbmapget(attrs, "NAME", -1, NULL)) != NULL){
+          cbmapput(map, tmp, -1, fbuf, flen, FALSE);
+          aname = cbsprintf("%s-filename", tmp);
+          if((tmp = cbmapget(attrs, "FILENAME", -1, NULL)) != NULL)
+            cbmapput(map, aname, -1, tmp, -1, FALSE);
+          free(aname);
+        }
+        free(fbuf);
+        cbmapclose(attrs);
       }
-      free(key);
+      cblistclose(parts);
+      free(bound);
+    } else {
+      pairs = cbsplit(buf, -1, "&");
+      for(i = 0; i < cblistnum(pairs); i++){
+        key = cbmemdup(cblistval(pairs, i, NULL), -1);
+        if((val = strchr(key, '=')) != NULL){
+          *(val++) = '\0';
+          dkey = cburldecode(key, NULL);
+          dval = cburldecode(val, NULL);
+          cbmapput(map, dkey, -1, dval, -1, FALSE);
+          free(dval);
+          free(dkey);
+        }
+        free(key);
+      }
+      cblistclose(pairs);
     }
-    cblistclose(pairs);
-    free(buf);
   }
-  free(rbuf);
+  free(buf);
   return map;
 }
 
@@ -681,7 +795,7 @@ void senddump(void){
 /* send Atom feed */
 void sendfeed(void){
   ARTICLE *art;
-  char *tmp, *prefix, *xml;
+  char *tmp, *xml;
   int snum;
   if(!g_rbbs){
     sendnotfound();
@@ -711,7 +825,6 @@ void sendfeed(void){
   xprintf("<?xml version=\"1.0\" encoding=\"%@\"?>\n", ENCODING);
   xprintf("<feed version=\"0.3\" xmlns=\"http://purl.org/atom/ns#\" xml:lang=\"%@\">\n",
           g_pagelang);
-  prefix = cbsprintf("%s?id=", g_systemuri);
   if(art){
     xprintf("<link rel=\"alternate\" type=\"%@\" href=\"%@?id=%@\"/>\n",
             HTMLTYPE, g_systemuri, artid(art));
@@ -722,8 +835,8 @@ void sendfeed(void){
     tmp = dateforwww(artmdate(art), FALSE);
     xprintf("<modified>%@</modified>\n", tmp);
     free(tmp);
-    xprintf("<generator name=\"RBBS\">%@?version=%?</generator>\n", RBBS_URI, RBBS_VERSION);
-    xml = arttoatom(art, prefix, g_blknum, g_resnum);
+    xprintf("<generator>RBBS %@</generator>\n", RBBS_VERSION);
+    xml = arttoatom(art, g_systemuri, g_blknum, g_resnum);
     printf("%s", xml);
     free(xml);
     freearticle(art);
@@ -737,11 +850,11 @@ void sendfeed(void){
     xprintf("<modified>%@</modified>\n", tmp);
     free(tmp);
     xprintf("<tagline>%@</tagline>\n", g_pagesummary);
-    xprintf("<generator name=\"RBBS\">%@?version=%?</generator>\n", RBBS_URI, RBBS_VERSION);
+    xprintf("<generator>RBBS %@</generator>\n", RBBS_VERSION);
     snum = 0;
     while(snum < g_feednum && (tmp = getnextid(g_rbbs)) != NULL){
       if((art = getarticle(g_rbbs, tmp)) != NULL){
-        xml = arttoatom(art, prefix, g_blknum, g_resnum);
+        xml = arttoatom(art, g_systemuri, g_blknum, g_resnum);
         printf("%s", xml);
         free(xml);
         freearticle(art);
@@ -750,11 +863,83 @@ void sendfeed(void){
       free(tmp);
     }
   }
-  free(prefix);
   xprintf("</feed>\n");
 }
 
 
+/* send as-is file */
+void sendasis(const char *id){
+  CBMAP *attrs;
+  const char *filename;
+  char *buf, *tmp;
+  int i, size;
+  if(!g_rbbs) senderror("The database is broken.");
+  attrs = cbmapopenex(PETITBNUM);
+  if(!(buf = loadfile(g_rbbs, id, &size, attrs))){
+    cbmapclose(attrs);
+    senderror("The file was not found.");
+  }
+  if(!(filename = cbmapget(attrs, "subject", -1, NULL))) filename = "";
+  printf("Cache-Control: no-cache, must-revalidate\r\n");
+  printf("Pragma: no-cache\r\n");
+  if(filename[0] != '\0'){
+    tmp = cbmimeencode(filename, "UTF-8", TRUE);
+    printf("Content-Disposition: inline; filename=%s\r\n", tmp);
+    free(tmp);
+  }
+  printf("Content-Length: %d\r\n", size);
+  printf("Content-Type: %s\r\n", getmimetype(filename));
+  printf("\r\n");
+  for(i = 0; i < size; i++){
+    putchar(buf[i]);
+  }
+  free(buf);
+  cbmapclose(attrs);
+}
+
+
+/* Get the MIME type of a file. */
+const char *getmimetype(const char *name){
+  if(!cbstricmp(name, "README")) return "text/plain";
+  if(!cbstricmp(name, "COPYING")) return "text/plain";
+  if(!cbstricmp(name, "ChangeLog")) return "text/plain";
+  if(cbstrbwimatch(name, ".txt")) return "text/plain";
+  if(cbstrbwimatch(name, ".asc")) return "text/plain";
+  if(cbstrbwimatch(name, ".html")) return "text/html";
+  if(cbstrbwimatch(name, ".htm")) return "text/html";
+  if(cbstrbwimatch(name, ".css")) return "text/css";
+  if(cbstrbwimatch(name, ".js")) return "text/javascript";
+  if(cbstrbwimatch(name, ".tsv")) return "text/tab-separated-values";
+  if(cbstrbwimatch(name, ".png")) return "image/png";
+  if(cbstrbwimatch(name, ".jpg")) return "image/jpeg";
+  if(cbstrbwimatch(name, ".jpeg")) return "image/jpeg";
+  if(cbstrbwimatch(name, ".gif")) return "image/gif";
+  if(cbstrbwimatch(name, ".tif")) return "image/tiff";
+  if(cbstrbwimatch(name, ".tiff")) return "image/tiff";
+  if(cbstrbwimatch(name, ".bmp")) return "image/bmp";
+  if(cbstrbwimatch(name, ".svg")) return "image/svg+xml";
+  if(cbstrbwimatch(name, ".xbm")) return "image/x-xbitmap";
+  if(cbstrbwimatch(name, ".au")) return "audio/basic";
+  if(cbstrbwimatch(name, ".snd")) return "audio/basic";
+  if(cbstrbwimatch(name, ".mid")) return "audio/midi";
+  if(cbstrbwimatch(name, ".midi")) return "audio/midi";
+  if(cbstrbwimatch(name, ".mp3")) return "audio/mpeg";
+  if(cbstrbwimatch(name, ".mpg")) return "video/mpeg";
+  if(cbstrbwimatch(name, ".mpeg")) return "video/mpeg";
+  if(cbstrbwimatch(name, ".xml")) return "application/xml";
+  if(cbstrbwimatch(name, ".zip")) return "application/zip";
+  if(cbstrbwimatch(name, ".rtf")) return "application/rtf";
+  if(cbstrbwimatch(name, ".pdf")) return "application/pdf";
+  if(cbstrbwimatch(name, ".ps")) return "application/postscript";
+  if(cbstrbwimatch(name, ".eps")) return "application/postscript";
+  if(cbstrbwimatch(name, ".doc")) return "application/msword";
+  if(cbstrbwimatch(name, ".xls")) return "application/vnd.ms-excel";
+  if(cbstrbwimatch(name, ".ppt")) return "application/vnd.ms-powerpoint";
+  if(cbstrbwimatch(name, ".xdw")) return "application/vnd.fujixerox.docuworks";
+  return "application/octet-stream";
+}
+
+
 /* send HTML page */
 void sendhtml(void){
   const char *fstr;
@@ -772,6 +957,11 @@ void sendhtml(void){
     printf("Last-Modified: %s\r\n", tmp);
     free(tmp);
   }
+  if(p_userpass[0] != '\0'){
+    tmp = cburlencode(p_userpass, -1);
+    printf("Set-Cookie: userpass=%s\r\n", tmp);
+    free(tmp);
+  }
   if((p_action == ACTPOSTART || p_action == ACTPOSTRES) && p_author[0] != '\0'){
     tmp = cburlencode(p_author, -1);
     printf("Set-Cookie: author=%s; expires=Thu, 1-Jan-2037 00:00:00 GMT\r\n", tmp);
@@ -856,6 +1046,15 @@ void sendhtml(void){
     case ACTDELEDB:
       deletedb();
       break;
+    case ACTLISTFILE:
+    case ACTSAVEFILE:
+    case ACTDELEFILE:
+      mylistfiles();
+      break;
+    case ACTSHOWMEMO:
+    case ACTPOSTMEMO:
+      showmemo();
+      break;
     case ACTHELP:
       showhelp();
       break;
@@ -878,9 +1077,13 @@ void sendhtml(void){
 /* check whether the user can administrate the system */
 int canadmin(void){
   if(g_adminpass[0] == '\0') return TRUE;
-  if(cbstrfwmatch(g_adminpass, CRPREFIX))
-    return matchcrypt(p_userpass, g_adminpass + strlen(CRPREFIX));
+  if(cbstrfwmatch(g_adminpass, CRPREFIX)){
+    if(matchcrypt(p_userpass, g_adminpass + strlen(CRPREFIX))) return TRUE;
+    if(matchcrypt(c_userpass, g_adminpass + strlen(CRPREFIX))) return TRUE;
+    return FALSE;
+  }
   if(!strcmp(p_userpass, g_adminpass)) return TRUE;
+  if(!strcmp(c_userpass, g_adminpass)) return TRUE;
   return FALSE;
 }
 
@@ -889,9 +1092,13 @@ int canadmin(void){
 int canwrite(void){
   if(canadmin()) return TRUE;
   if(g_writepass[0] == '\0') return TRUE;
-  if(cbstrfwmatch(g_writepass, CRPREFIX))
-    return matchcrypt(p_userpass, g_writepass + strlen(CRPREFIX));
+  if(cbstrfwmatch(g_writepass, CRPREFIX)){
+    if(matchcrypt(p_userpass, g_writepass + strlen(CRPREFIX))) return TRUE;
+    if(matchcrypt(c_userpass, g_writepass + strlen(CRPREFIX))) return TRUE;
+    return FALSE;
+  }
   if(!strcmp(p_userpass, g_writepass)) return TRUE;
+  if(!strcmp(c_userpass, g_writepass)) return TRUE;
   return FALSE;
 }
 
@@ -900,13 +1107,27 @@ int canwrite(void){
 int canartadmin(void){
   ARTICLE *art;
   int match;
-  if(!g_rbbs || !(art = getarticle(g_rbbs, p_id))) return FALSE;
-  match = g_postauth && matchcrypt(p_userpass, artpassword(art));
+  if(!g_rbbs || !g_postauth || !(art = getarticle(g_rbbs, p_id))) return FALSE;
+  match = matchcrypt(p_userpass, artpassword(art));
   freearticle(art);
   return match;
 }
 
 
+/* check whether the user can administrate the file */
+int canfileadmin(void){
+  CBMAP *attrs;
+  const char *tmp;
+  int match;
+  attrs = cbmapopenex(PETITBNUM);
+  if(!g_rbbs || !g_postauth) return FALSE;
+  loadfile(g_rbbs, p_id, NULL, attrs);
+  match = (tmp = cbmapget(attrs, "password", -1, NULL)) && matchcrypt(p_userpass, tmp);
+  cbmapclose(attrs);
+  return match;
+}
+
+
 /* check the password and show the form to input a password */
 int authorize(void){
   int admin, artadm;
@@ -919,6 +1140,8 @@ int authorize(void){
   case ACTSEARCH:
   case ACTHELP:
   case ACTPOSTRES:
+  case ACTSHOWMEMO:
+  case ACTPOSTMEMO:
     return TRUE;
   case ACTARTFORM:
   case ACTARTCNFM:
@@ -932,17 +1155,26 @@ int authorize(void){
   case ACTDELEDB:
     admin = TRUE;
     break;
+  case ACTLISTFILE:
+  case ACTSAVEFILE:
+    break;
+  case ACTDELEFILE:
+    admin = TRUE;
+    break;
   default:
     break;
   }
   if(admin){
     if(canadmin()) return TRUE;
+    if(p_action == ACTDELEFILE && canfileadmin()) return TRUE;
     if(artadm && canartadmin()) return TRUE;
   } else {
     if(canwrite()) return TRUE;
   }
   if(p_userpass[0] != '\0'){
     shownote("The password did not match.  Try again.");
+  } else if(p_action == ACTDELEFILE && g_postauth){
+    shownote("The password of the file is needed.");
   } else if(artadm && g_postauth){
     shownote("The password of the article is needed.");
   } else if(admin){
@@ -1009,6 +1241,10 @@ void showpagenavi(void){
           g_systemuri, ACTHELP, ++g_tabidx, g_helplabel);
   xprintf("<a href=\"%@?action=%d\" class=\"navianc\" tabindex=\"%d\" accesskey=\"9\">%@</a>\n",
           g_systemuri, ACTARTFORM, ++g_tabidx, g_postlabel);
+  xprintf("<a href=\"%@?action=%d\" class=\"navianc\" tabindex=\"%d\">%@</a>\n",
+          g_systemuri, ACTLISTFILE, ++g_tabidx, g_filelabel);
+  xprintf("<a href=\"%@?action=%d\" class=\"navianc\" tabindex=\"%d\">%@</a>\n",
+          g_systemuri, ACTSHOWMEMO, ++g_tabidx, g_memolabel);
   fstr = FEEDCDATE;
   if(g_sort == SORTMDATEA || g_sort == SORTMDATED) fstr = FEEDMDATE;
   xprintf("<a href=\"%@?feed=%@\" type=\"%@\" class=\"navianc\" tabindex=\"%d\">%@</a>\n",
@@ -1073,8 +1309,8 @@ int matchingart(ARTICLE *art){
   int i, hit;
   if(p_language[0] != '\0' && strcmp(artlanguage(art), p_language)) return FALSE;
   if(p_category[0] != '\0' && strcmp(artcategory(art), p_category)) return FALSE;
-  if(p_subject[0] != '\0' && !strstr(artsubject(art), p_subject)) return FALSE;
-  if(p_author[0] != '\0' && !strstr(artauthor(art), p_author)) return FALSE;
+  if(p_subject[0] != '\0' && strcmp(artsubject(art), p_subject)) return FALSE;
+  if(p_author[0] != '\0' && strcmp(artauthor(art), p_author)) return FALSE;
   if(p_text[0] != '\0'){
     text = arttotext(art);
     words = cbsplit(p_text, -1, " \t\n");
@@ -1423,7 +1659,8 @@ void showartwithctl(ARTICLE *art){
     xprintf("<span class=\"navivoid\">%@</span>\n", g_responselabel);
   }
   xprintf("</div>\n");
-  html = arttohtml(art, p_focus != FCSSIMPLE ? 0 : g_blknum, p_focus == FCSDETAIL ? 0 : g_resnum);
+  html = arttohtml(art, g_systemuri, p_focus != FCSSIMPLE ? 0 : g_blknum,
+                   p_focus == FCSDETAIL ? 0 : g_resnum);
   xprintf("%s", html);
   free(html);
   if(p_focus == FCSSIMPLE && !strchr(artflags(art), LONECHAR)){
@@ -1532,7 +1769,8 @@ void showartform(void){
     xprintf("<td class=\"tlabel\">%@:</td>\n", g_subjectlabel);
     xprintf("<td class=\"tbody\">\n");
     xprintf("<input type=\"text\" name=\"subject\" value=\"%@\" size=\"48\""
-            " class=\"text\" tabindex=\"%d\" />\n", p_subject, ++g_tabidx);
+            " class=\"text\" tabindex=\"%d\" />\n",
+            p_subject[0] != '\0' ? p_subject : g_defsubject, ++g_tabidx);
     xprintf("</td>\n");
     xprintf("</tr>\n");
     xprintf("<tr>\n");
@@ -1542,7 +1780,8 @@ void showartform(void){
       xprintf("%@\n", a_author);
     } else {
       xprintf("<input type=\"text\" name=\"author\" value=\"%@\" size=\"16\""
-              " class=\"text\" tabindex=\"%d\" />\n", p_author, ++g_tabidx);
+              " class=\"text\" tabindex=\"%d\" />\n",
+              p_author[0] != '\0' ? p_author : g_defauthor, ++g_tabidx);
     }
     xprintf("</td>\n");
     xprintf("</tr>\n");
@@ -1634,6 +1873,8 @@ void showartconfirm(void){
             plaincat(buf, cline + 1, FALSE);
             xsprintf(buf, "\n");
             if(nline[0] != ASISCHAR) xsprintf(buf, "</asis>\n");
+          } else if(!strcmp(cline, BREAKSTR)){
+            xsprintf(buf, "<break/>\n");
           } else {
             xsprintf(buf, "<para>");
             plaincat(buf, cline, TRUE);
@@ -1684,7 +1925,7 @@ void showartconfirm(void){
       shownote("Warning: The password is empty and everyone can edit the article.");
     xml = arttoxml(art);
     xprintf("<div class=\"artctl\">\n");
-    tmp = arttohtml(art, 0, 0);
+    tmp = arttohtml(art, g_systemuri, 0, 0);
     xprintf("%s", tmp);
     free(tmp);
     xprintf("</div>\n");
@@ -1859,7 +2100,7 @@ void mypostarticle(void){
   } else if(!postarticle(g_rbbs, art, p_id[0] != '\0' ? p_id : NULL)){
     shownote(rbbsemsg(g_rbbs));
   } else {
-    shownote("Posting the article have been completed.");
+    shownote("Posting the article has been completed.");
     xprintf("<div class=\"todo\">\n");
     xprintf("<a href=\"%@\" class=\"navianc\" tabindex=\"%d\">%@</a>\n",
             g_systemuri, ++g_tabidx, g_reloadlabel);
@@ -1880,7 +2121,7 @@ void mydeletearticle(void){
   if(!deletearticle(g_rbbs, p_id)){
     shownote(rbbsemsg(g_rbbs));
   } else {
-    shownote("Deleting the article have been completed.");
+    shownote("Deleting the article has been completed.");
     xprintf("<div class=\"todo\">\n");
     xprintf("<a href=\"%@\" class=\"navianc\" tabindex=\"%d\">%@</a>\n",
             g_systemuri, ++g_tabidx, g_reloadlabel);
@@ -1945,6 +2186,170 @@ void deletedb(void){
 }
 
 
+/* list the files */
+void mylistfiles(void){
+  CBLIST *list;
+  CBMAP *attrs;
+  const char *name, *tmp, *id;
+  char *date;
+  int i;
+  if(!g_rbbs) return;
+  if(p_action == ACTSAVEFILE && p_filelen > 0){
+    name = p_filename;
+    if((tmp = strrchr(name, '\\')) != NULL) name = tmp + 1;
+    if((tmp = strrchr(name, '/')) != NULL) name = tmp + 1;
+    if(name[0] == '\0'){
+      shownote("The subject of the file is needed.");
+    } else if(p_author[0] == '\0'){
+      shownote("The author of the file is needed.");
+    } else if(savefile(g_rbbs, p_password, name, p_author, g_curdate, p_file, p_filelen)){
+      shownote("A new file was saved.");
+      if(g_postauth && p_password[0] == '\0')
+        shownote("Warning: The password is empty and everyone can delete the file.");
+    } else {
+      shownote("Saving was failed.");
+    }
+  } else if(p_action == ACTDELEFILE && p_id[0] != '\0'){
+    if(deletefile(g_rbbs, p_id)){
+      shownote("A file was deleted.");
+    } else {
+      shownote("Deleting was failed.");
+    }
+  }
+  xprintf("<form method=\"post\" enctype=\"multipart/form-data\" action=\"%s\">\n", g_systemuri);
+  xprintf("<div class=\"comform\">\n");
+  xprintf("<table summary=\"form table\">\n");
+  xprintf("<tr>\n");
+  xprintf("<td class=\"tlabel\">%@:</td>\n", g_pathlabel);
+  xprintf("<td class=\"tbody\">\n");
+  xprintf("<input type=\"file\" name=\"file\" size=\"64\" class=\"file\" tabindex=\"%d\" />\n",
+          ++g_tabidx);
+  xprintf("</td>\n");
+  xprintf("</tr>\n");
+  xprintf("<tr>\n");
+  xprintf("<td class=\"tlabel\">%@:</td>\n", g_subjectlabel);
+  xprintf("<td class=\"tbody\">\n");
+  xprintf("<input type=\"text\" name=\"filename\" value=\"\" size=\"32\""
+          " class=\"text\" tabindex=\"%d\" />\n", ++g_tabidx);
+  xprintf("</td>\n");
+  xprintf("</tr>\n");
+  xprintf("<tr>\n");
+  xprintf("<td class=\"tlabel\">%@:</td>\n", g_authorlabel);
+  xprintf("<td class=\"tbody\">\n");
+  xprintf("<input type=\"text\" name=\"author\" value=\"%@\" size=\"16\""
+          " class=\"text\" tabindex=\"%d\" />\n", p_author, ++g_tabidx);
+  xprintf("</td>\n");
+  xprintf("</tr>\n");
+  if(g_postauth){
+    xprintf("<tr>\n");
+    xprintf("<td class=\"tlabel\">%@:</td>\n", g_passwordlabel);
+    xprintf("<td class=\"tbody\">\n");
+    xprintf("<input type=\"password\" name=\"password\" value=\"%@\" size=\"16\""
+            " class=\"password\" tabindex=\"%d\" />\n", p_password, ++g_tabidx);
+    xprintf("</td>\n");
+    xprintf("</tr>\n");
+  }
+  xprintf("<tr>\n");
+  xprintf("<td class=\"tlabel\"></td>\n");
+  xprintf("<td class=\"tbody\"><input type=\"submit\" value=\"%@\" class=\"submit\""
+          " tabindex=\"%d\" /></td>\n", g_submitlabel, ++g_tabidx);
+  xprintf("</tr>\n");
+  xprintf("</table>\n");
+  xprintf("</div>\n");
+  xprintf("<div class=\"hidden\">\n");
+  xprintf("<input type=\"hidden\" name=\"action\" value=\"%d\" />\n", ACTSAVEFILE);
+  xprintf("<input type=\"hidden\" name=\"userpass\" value=\"%@\" />\n", p_userpass);
+  xprintf("</div>\n");
+  xprintf("</form>\n");
+  xprintf("<div class=\"flist\">\n");
+  xprintf("<table summary=\"form table\">\n");
+  xprintf("<tr>\n");
+  xprintf("<td class=\"flabel\">%@</td>\n", g_subjectlabel);
+  xprintf("<td class=\"flabel\">%@</td>\n", g_authorlabel);
+  xprintf("<td class=\"flabel\">%@</td>\n", g_cdatelabel);
+  xprintf("<td class=\"flabel\">%@</td>\n", g_sizelabel);
+  xprintf("<td class=\"flabel\">%@</td>\n", g_typelabel);
+  xprintf("<td class=\"flabel\">%@</td>\n", g_actionlabel);
+  xprintf("</tr>\n");
+  list = listfiles(g_rbbs);
+  attrs = cbmapopenex(PETITBNUM);
+  for(i = 0; i < cblistnum(list); i++){
+    id = cblistval(list, cblistnum(list) - i - 1, NULL);
+    loadfile(g_rbbs, id, NULL, attrs);
+    date = dateformen(cbmapget(attrs, "cdate", -1, NULL));
+    xprintf("<tr>\n");
+    xprintf("<td class=\"fbody\">%@</td>\n", cbmapget(attrs, "subject", -1, NULL));
+    xprintf("<td class=\"fbody\">%@</td>\n", cbmapget(attrs, "author", -1, NULL));
+    xprintf("<td class=\"fbody\">%@</td>\n", date);
+    xprintf("<td class=\"fbody\">%@</td>\n", cbmapget(attrs, "size", -1, NULL));
+    xprintf("<td class=\"fbody\">%@</td>\n", getmimetype(cbmapget(attrs, "subject", -1, NULL)));
+    xprintf("<td class=\"fbody\">");
+    xprintf("<a href=\"%@/%?\" class=\"actanc\" tabindex=\"%d\">%@</a> ",
+            g_systemuri, id, ++g_tabidx, g_getlabel);
+    xprintf("<a href=\"%@?action=%d&amp;id=%?\" class=\"actanc\" tabindex=\"%d\">%@</a>\n",
+            g_systemuri, ACTDELEFILE, id, ++g_tabidx, g_dellabel);
+    xprintf("</td>\n");
+    xprintf("</tr>\n");
+    free(date);
+  }
+  cbmapclose(attrs);
+  cblistclose(list);
+  xprintf("</table>\n");
+  xprintf("</div>\n");
+}
+
+
+/* show the memorandum */
+void showmemo(void){
+  char *value;
+  if(!g_rbbs) return;
+  if(p_action == ACTPOSTMEMO){
+    if(g_writepass[0] != '\0' && !canwrite()){
+      shownote("The password did not match.  Try again.");
+    } else if(setattribute(g_rbbs, MEMOATTR, p_text)){
+      shownote("The memorandum was updated.");
+    } else {
+      shownote("The memorandum could not be updated.");
+    }
+  }
+  if(!(value = getattribute(g_rbbs, MEMOATTR))) value = cbmemdup("", 0);
+  xprintf("<form method=\"post\" action=\"%@\">\n", g_systemuri);
+  xprintf("<div class=\"comform\">\n");
+  xprintf("<table summary=\"form table\">\n");
+  xprintf("<tr>\n");
+  xprintf("<td class=\"tlabel\">%@:</td>\n", g_textlabel);
+  xprintf("<td class=\"tbody\">");
+  xprintf("<textarea name=\"text\" cols=\"80\" rows=\"20\" tabindex=\"%d\">%@</textarea>\n",
+          ++g_tabidx, p_action == ACTPOSTMEMO ? p_text : value);
+  xprintf("</td>\n");
+  xprintf("</tr>\n");
+  if(g_writepass[0] != '\0'){
+    xprintf("<tr>\n");
+    xprintf("<td class=\"tlabel\">%@:</td>\n", g_passwordlabel);
+    xprintf("<td class=\"tbody\">\n");
+    xprintf("<input type=\"password\" name=\"userpass\" value=\"%@\" size=\"16\""
+            " class=\"password\" tabindex=\"%d\" />\n", p_userpass, ++g_tabidx);
+    xprintf("</td>\n");
+  }
+  xprintf("<tr>\n");
+  xprintf("<td class=\"tlabel\"></td>\n");
+  xprintf("<td class=\"tbody\">");
+  xprintf("<input type=\"submit\" value=\"%@\" class=\"submit\" tabindex=\"%d\" />",
+          g_submitlabel, ++g_tabidx);
+  xprintf("<input type=\"reset\" value=\"%@\" class=\"reset\" tabindex=\"%d\" />",
+          g_resetlabel, ++g_tabidx);
+  xprintf("</td>\n");
+  xprintf("</tr>\n");
+  xprintf("</table>\n");
+  xprintf("</div>\n");
+  xprintf("<div class=\"hidden\">\n");
+  xprintf("<input type=\"hidden\" name=\"action\" value=\"%d\" />\n", ACTPOSTMEMO);
+  xprintf("</div>\n");
+  xprintf("</form>\n");
+  free(value);
+}
+
+
 /* show the help file */
 void showhelp(void){
   ARTICLE *art;
@@ -1961,7 +2366,7 @@ void showhelp(void){
     return;
   }
   xprintf("<div class=\"artctl\">\n");
-  html = arttohtml(art, 0, 0);
+  html = arttohtml(art, g_systemuri, 0, 0);
   xprintf("%s", html);
   free(html);
   xprintf("</div>\n");
