@@ -1,6 +1,6 @@
 /*************************************************************************************************
  * Implementation of common features
- *                                                      Copyright (C) 2003-2004 Mikio Hirabayashi
+ *                                                      Copyright (C) 2003-2005 Mikio Hirabayashi
  * This file is part of RBBS, a web-based bulletin board system.
  * RBBS is free software; you can redistribute it and/or modify it under the terms of the GNU
  * General Public License as published by the Free Software Foundation; either version 2 of the
@@ -19,7 +19,7 @@
 
 /* private function prototypes */
 static int validxmlent(const char *xml);
-static char *uniqueid(void);
+static char *uniqueid(const char *prefix);
 static int validid(const char *id);
 static char *datestr(time_t t);
 static int validdate(const char *date);
@@ -27,7 +27,7 @@ static const char *wdayname(int wday);
 static const char *monname(int mon);
 static int strisprintable(const char *str);
 static char *headtohtml(ARTICLE *art);
-static char *bodytohtml(ARTICLE *art, int num);
+static char *bodytohtml(ARTICLE *art, const char *systemuri, int num);
 static char *tailtohtml(ARTICLE *art, int num);
 const char *artsubclass(const char *flags);
 const char *ressubclass(const char *flags);
@@ -43,8 +43,9 @@ static const char *hashclass(const char *prefix, const char *str);
 /* Get a handle connected to the databases. */
 RBBS *rbbsopen(const char *name, int writer){
   RBBS *rbbs;
-  DEPOT *artdb;
+  DEPOT *artdb, *fileidx;
   VILLA *cdidx, *mdidx;
+  CURIA *filedb;
   time_t t;
   char path[PATHBUFSIZ], *date;
   assert(name);
@@ -58,7 +59,13 @@ RBBS *rbbsopen(const char *name, int writer){
   cdidx = vlopen(path, writer ? (VL_OWRITER | VL_OCREAT) : VL_OREADER, VL_CMPLEX);
   sprintf(path, "%s/%s", name, MDIDXNAME);
   mdidx = vlopen(path, writer ? (VL_OWRITER | VL_OCREAT) : VL_OREADER, VL_CMPLEX);
-  if(!artdb || !cdidx || !mdidx){
+  sprintf(path, "%s/%s", name, FILEDBNAME);
+  filedb = cropen(path, writer ? (CR_OWRITER | CR_OCREAT) : CR_OREADER, -1, -1);
+  sprintf(path, "%s/%s", name, FILEIDXNAME);
+  fileidx = dpopen(path, writer ? (DP_OWRITER | DP_OCREAT) : DP_OREADER, -1);
+  if(!artdb || !cdidx || !mdidx || !filedb){
+    if(fileidx) dpclose(fileidx);
+    if(filedb) crclose(filedb);
     if(mdidx) vlclose(mdidx);
     if(cdidx) vlclose(cdidx);
     if(artdb) dpclose(artdb);
@@ -78,6 +85,8 @@ RBBS *rbbsopen(const char *name, int writer){
   rbbs->mdidx = mdidx;
   rbbs->idx = NULL;
   rbbs->next = NULL;
+  rbbs->filedb = filedb;
+  rbbs->fileidx = fileidx;
   if(!(rbbs->cdate = dpget(artdb, CDATEKEY, -1, 0, -1, NULL))) rbbs->cdate = datestr(t);
   if(!(rbbs->mdate = dpget(artdb, MDATEKEY, -1, 0, -1, NULL))) rbbs->mdate = datestr(t);
   rbbs->errmsg = NULL;
@@ -93,6 +102,8 @@ int rbbsclose(RBBS *rbbs){
   free(rbbs->errmsg);
   free(rbbs->mdate);
   free(rbbs->cdate);
+  if(!dpclose(rbbs->fileidx)) err = TRUE;
+  if(!crclose(rbbs->filedb)) err = TRUE;
   if(!vlclose(rbbs->mdidx)) err = TRUE;
   if(!vlclose(rbbs->cdidx)) err = TRUE;
   if(!dpclose(rbbs->artdb)) err = TRUE;
@@ -320,6 +331,107 @@ int deletearticle(RBBS *rbbs, const char *id){
 }
 
 
+/* Get an attribute of a database. */
+char *getattribute(RBBS *rbbs, const char *name){
+  assert(rbbs && name);
+  if(cbstrfwmatch(name, AIDPREFIX)) return NULL;
+  return dpget(rbbs->artdb, name, -1, 0, -1, NULL);
+}
+
+
+/* Set an attribute of a database. */
+int setattribute(RBBS *rbbs, const char *name, const char *value){
+  assert(rbbs && name);
+  if(cbstrfwmatch(name, AIDPREFIX)) return FALSE;
+  if(value){
+    return dpput(rbbs->artdb, name, -1, value, -1, DP_DOVER);
+  } else {
+    return dpout(rbbs->artdb, name, -1);
+  }
+}
+
+
+/* Save a file into a database. */
+int savefile(RBBS *rbbs, const char *password, const char *subject, const char *author,
+             const char *cdate, const char *ptr, int size){
+  CBMAP *attrs;
+  char *id, *mbuf, numbuf[NUMBUFSIZ];
+  int err, mbsiz;
+  assert(rbbs && password && subject && author && cdate && ptr && size >= 0);
+  id = uniqueid(FIDPREFIX);
+  err = FALSE;
+  attrs = cbmapopenex(1);
+  cbmapput(attrs, "password", -1, makecrypt(password), -1, TRUE);
+  cbmapput(attrs, "subject", -1, subject, -1, TRUE);
+  cbmapput(attrs, "author", -1, author, -1, TRUE);
+  cbmapput(attrs, "cdate", -1, cdate, -1, TRUE);
+  sprintf(numbuf, "%d", size);
+  cbmapput(attrs, "size", -1, numbuf, -1, TRUE);
+  mbuf = cbmapdump(attrs, &mbsiz);
+  if(!dpput(rbbs->fileidx, id, -1, mbuf, mbsiz, FALSE)) err = TRUE;
+  if(!crput(rbbs->filedb, id, -1, ptr, size, FALSE)) err = TRUE;
+  free(mbuf);
+  cbmapclose(attrs);
+  free(id);
+  return err ? FALSE : TRUE;
+}
+
+
+/* Load a file in a database. */
+char *loadfile(RBBS *rbbs, const char *id, int *sp, CBMAP *attrs){
+  CBMAP *map;
+  const char *tmp;
+  char *mbuf;
+  int mbsiz;
+  assert(rbbs && id);
+  if(!(mbuf = dpget(rbbs->fileidx, id, -1, 0, -1, &mbsiz))) return NULL;
+  if(attrs){
+    map = cbmapload(mbuf, mbsiz);
+    if(!(tmp = cbmapget(map, "password", -1, NULL))) tmp = "";
+    cbmapput(attrs, "password", -1, tmp, -1, TRUE);
+    if(!(tmp = cbmapget(map, "subject", -1, NULL))) tmp = "";
+    cbmapput(attrs, "subject", -1, tmp, -1, TRUE);
+    if(!(tmp = cbmapget(map, "author", -1, NULL))) tmp = "";
+    cbmapput(attrs, "author", -1, tmp, -1, TRUE);
+    if(!(tmp = cbmapget(map, "cdate", -1, NULL))) tmp = "";
+    cbmapput(attrs, "cdate", -1, tmp, -1, TRUE);
+    if(!(tmp = cbmapget(map, "size", -1, NULL))) tmp = "";
+    cbmapput(attrs, "size", -1, tmp, -1, TRUE);
+    cbmapclose(map);
+    free(mbuf);
+  }
+  if(!sp) return NULL;
+  return crget(rbbs->filedb, id, -1, 0, -1, sp);
+}
+
+
+/* Save a file into a database. */
+int deletefile(RBBS *rbbs, const char *id){
+  int err;
+  assert(rbbs && id);
+  err = FALSE;
+  if(!dpout(rbbs->fileidx, id, -1)) err = TRUE;
+  if(!crout(rbbs->filedb, id, -1)) err = TRUE;
+  return err ? FALSE : TRUE;
+}
+
+
+/* Get a list of all files in a database. */
+CBLIST *listfiles(RBBS *rbbs){
+  CBLIST *list;
+  char *buf;
+  assert(rbbs);
+  list = cblistopen();
+  criterinit(rbbs->filedb);
+  while((buf = criternext(rbbs->filedb, NULL)) != NULL){
+    cblistpush(list, buf, -1);
+    free(buf);
+  }
+  cblistsort(list);
+  return list;
+}
+
+
 /* Make a object of article from an XML string. */
 ARTICLE *makearticle(const char *xml){
   ARTICLE *art;
@@ -558,6 +670,14 @@ ARTICLE *makearticle(const char *xml){
           } else {
             cblistpush(art->body, elem, -1);
           }
+        } else if(!strcmp(name, "break")){
+          if(!pname || strcmp(pname, "body")){
+            art->errmsg = cbsprintf("Invalid-XML: \"break\" must be a child of \"body\"");
+          } else if(!cbstrbwmatch(elem, "/>")){
+            art->errmsg = cbsprintf("Invalid-XML: \"break\" must be empty");
+          } else {
+            cblistpush(art->body, elem, -1);
+          }
         } else if(!strcmp(name, "link")){
           if(!pname || bstart < 0){
             art->errmsg = cbsprintf("Invalid-XML: \"link\" must be in a block");
@@ -637,7 +757,7 @@ ARTICLE *makearticle(const char *xml){
     if(art->errmsg) break;
   }
   if(!art->id){
-    art->id = uniqueid();
+    art->id = uniqueid(AIDPREFIX);
   } else if(!validid(art->id)){
     if(!art->errmsg) art->errmsg = cbsprintf("Invalid-XML: the ID is invalid");
   }
@@ -864,6 +984,8 @@ char *arttoxml(ARTICLE *art){
           if(!strcmp(name, "graph")){
             if(!(val = cbmapget(attrs, "data", -1, NULL))) val = "";
             xsprintf(buf, "<graph data=\"%@\"/>\n", val);
+          } else if(!strcmp(name, "break")){
+            xsprintf(buf, "<break/>\n");
           } else if(!strcmp(name, "link")){
             if(!(val = cbmapget(attrs, "to", -1, NULL))) val = "";
             xsprintf(buf, "<link to=\"%@\">", val);
@@ -945,10 +1067,20 @@ char *arttotext(ARTICLE *art){
     elems = cbxmlbreak(blk, FALSE);
     for(j = 0; j < cblistnum(elems); j++){
       elem = cblistval(elems, j, NULL);
-      if(elem[0] == '<') continue;
-      rtext = cbxmlunescape(elem);
-      xsprintf(buf, "%s", rtext);
-      free(rtext);
+      if(elem[0] == '<'){
+        attrs = cbxmlattrs(elem);
+        name = cbmapget(attrs, "", -1, NULL);
+        if(!strcmp(name, "graph")){
+          xsprintf(buf, "[graph]");
+        } else if(!strcmp(name, "break")){
+          xsprintf(buf, "------");
+        }
+        cbmapclose(attrs);
+      } else {
+        rtext = cbxmlunescape(elem);
+        xsprintf(buf, "%s", rtext);
+        free(rtext);
+      }
     }
     xsprintf(buf, "\n");
     cblistclose(elems);
@@ -996,17 +1128,17 @@ char *arttotext(ARTICLE *art){
 
 
 /* Serialize an article into a string of HTML. */
-char *arttohtml(ARTICLE *art, int bnum, int rnum){
+char *arttohtml(ARTICLE *art, const char *systemuri, int bnum, int rnum){
   CBDATUM *buf;
   char *tmp;
-  assert(art);
+  assert(art && systemuri);
   buf = cbdatumopen("", 0);
   xsprintf(buf, "<div id=\"%@\" lang=\"%@\" xml:lang=\"%@\" class=\"article%s\">\n",
            art->id, art->language, art->language, artsubclass(art->flags));
   tmp = headtohtml(art);
   xsprintf(buf, "%s", tmp);
   free(tmp);
-  tmp = bodytohtml(art, bnum);
+  tmp = bodytohtml(art, systemuri, bnum);
   xsprintf(buf, "%s", tmp);
   free(tmp);
   tmp = tailtohtml(art, rnum);
@@ -1018,14 +1150,15 @@ char *arttohtml(ARTICLE *art, int bnum, int rnum){
 
 
 /* Serialize an article into a string of Atom. */
-char *arttoatom(ARTICLE *art, const char *prefix, int bnum, int rnum){
+char *arttoatom(ARTICLE *art, const char *systemuri, int bnum, int rnum){
   CBDATUM *buf;
   char *tmp;
-  assert(art && prefix);
+  assert(art && systemuri);
   buf = cbdatumopen("", 0);
   xsprintf(buf, "<entry>\n");
-  xsprintf(buf, "<id>%@%@</id>\n", prefix, art->id);
-  xsprintf(buf, "<link rel=\"alternate\" type=\"text/html\" href=\"%@%@\"/>\n", prefix, art->id);
+  xsprintf(buf, "<id>%@?id=%@</id>\n", systemuri, art->id);
+  xsprintf(buf, "<link rel=\"alternate\" type=\"text/html\" href=\"%@?id=%@\"/>\n",
+           systemuri, art->id);
   xsprintf(buf, "<title>%@</title>\n", art->subject);
   xsprintf(buf, "<author>\n");
   xsprintf(buf, "<name>%@</name>\n", art->author);
@@ -1040,8 +1173,8 @@ char *arttoatom(ARTICLE *art, const char *prefix, int bnum, int rnum){
   xsprintf(buf, "<modified>%@</modified>\n", tmp);
   free(tmp);
   xsprintf(buf, "<content type=\"text/html\" mode=\"escaped\""
-           " xml:base=\"%@%s\" xml:lang=\"%@\">\n", prefix, art->id, art->language);
-  tmp = bodytohtml(art, bnum);
+           " xml:base=\"%@?id=%s\" xml:lang=\"%@\">\n", systemuri, art->id, art->language);
+  tmp = bodytohtml(art, systemuri, bnum);
   xsprintf(buf, "%@", tmp);
   free(tmp);
   tmp = tailtohtml(art, rnum);
@@ -1319,7 +1452,7 @@ static int validxmlent(const char *xml){
 
 /* Create a unique ID.
    The return value is the string of a unique ID. */
-static char *uniqueid(void){
+static char *uniqueid(const char *prefix){
   FILE *ifp;
   char str[PATHBUFSIZ];
   int i, rnd;
@@ -1331,7 +1464,7 @@ static char *uniqueid(void){
     fclose(ifp);
   }
   sprintf(str, "%s%010d%05d%05d",
-          IDPREFIX, (int)time(NULL), (int)getpid() % 100000, (unsigned int)rnd % 100000);
+          prefix, (int)time(NULL), (int)getpid() % 100000, (unsigned int)rnd % 100000);
   return cbmemdup(str, -1);
 }
 
@@ -1341,8 +1474,8 @@ static char *uniqueid(void){
 static int validid(const char *id){
   int i;
   assert(id);
-  if(!cbstrfwmatch(id, IDPREFIX)) return FALSE;
-  for(i = strlen(IDPREFIX); id[i] != '\0'; i++){
+  if(!cbstrfwmatch(id, AIDPREFIX)) return FALSE;
+  for(i = strlen(AIDPREFIX); id[i] != '\0'; i++){
     if(id[i] < '0' || id[i] > '9') return FALSE;
   }
   return TRUE;
@@ -1418,7 +1551,7 @@ static const char *wdayname(int wday){
 
 
 /* Get the name of a month.
-   `wday' specifies the number of a month (0-11).
+   `mon' specifies the number of a month (0-11).
    The return value is the strint of the name of the month. */
 static const char *monname(int mon){
   switch(mon){
@@ -1483,10 +1616,11 @@ static char *headtohtml(ARTICLE *art){
 
 /* Extract HTML from the body of an article.
    `art' specifies a handle of a article object.
+   `systemuri' specifies the URI of the generator of the HTML.
    `num' specifies the number of shown blocks in the body.  If it is zero, all responses are
    shown.  If it is negative, the absolute value is calculated and the lower blocks are shown.
    The return value is a string of HTML. */
-static char *bodytohtml(ARTICLE *art, int num){
+static char *bodytohtml(ARTICLE *art, const char *systemuri, int num){
   CBDATUM *buf;
   CBLIST *elems;
   CBMAP *attrs;
@@ -1528,6 +1662,8 @@ static char *bodytohtml(ARTICLE *art, int num){
           name = "li";
         } else if(!strcmp(name, "graph")){
           name = "img";
+        } else if(!strcmp(name, "break")){
+          name = "hr";
         } else if(!strcmp(name, "link")){
           name = "a";
         } else if(!strcmp(name, "emp")){
@@ -1553,6 +1689,8 @@ static char *bodytohtml(ARTICLE *art, int num){
             if(!(val = cbmapget(attrs, "data", -1, NULL))) val = "";
             xsprintf(buf, "<div class=\"%@\"><img src=\"%@\" alt=\"[graph]\" /></div>\n",
                      oname, val);
+          } else if(!strcmp(name, "hr")){
+            xsprintf(buf, "<div class=\"%@\"><code>------</code></div>\n", oname);
           } else if(!strcmp(name, "a")){
             if(!(val = cbmapget(attrs, "to", -1, NULL))) val = "";
             xsprintf(buf, "<a href=\"%@\" class=\"%@\">", val, oname);
