@@ -1,6 +1,6 @@
 /*************************************************************************************************
  * Interface of common features
- *                                                      Copyright (C) 2003-2004 Mikio Hirabayashi
+ *                                                      Copyright (C) 2003-2005 Mikio Hirabayashi
  * This file is part of RBBS, a web-based bulletin board system.
  * RBBS is free software; you can redistribute it and/or modify it under the terms of the GNU
  * General Public License as published by the Free Software Foundation; either version 2 of the
@@ -25,6 +25,7 @@
 
 
 #include <depot.h>
+#include <curia.h>
 #include <cabin.h>
 #include <villa.h>
 
@@ -52,7 +53,6 @@
 #ifndef RBBS_VERSION                     /* version and URI */
 #define RBBS_VERSION   "0.0.0"
 #endif
-#define RBBS_URI       "http://rbbs.sourceforge.jp/"
 
 #undef TRUE                              /* boolean true */
 #define TRUE           1
@@ -62,12 +62,15 @@
 #define ARTDBNAME      "artdb"           /* name of the database for articles */
 #define CDIDXNAME      "cdidx"           /* name of the index for modification date */
 #define MDIDXNAME      "mdidx"           /* name of the index for creation date */
+#define FILEDBNAME     "filedb"          /* name of the detabase for files */
+#define FILEIDXNAME    "fileidx"         /* name of the detabase for file attributes */
 #define CDATEKEY       "cdate"           /* key of the creation date */
 #define MDATEKEY       "mdate"           /* key of the modification date */
 #define PATHBUFSIZ     2048              /* size of a path buffer */
 #define NUMBUFSIZ      32                /* size of a number buffer */
 #define DEFJETLAG      1440              /* default value of the jet lag */
-#define IDPREFIX       "RA"              /* prefix of article ID */
+#define AIDPREFIX      "RA"              /* prefix of article ID */
+#define FIDPREFIX      "RF"              /* prefix of file ID */
 #define HASHNUM        7                 /* number of hash classes */
 #define LONESTR        "lone"            /* string for lone articles */
 #define LONECHAR       'l'               /* character for lone articles */
@@ -87,6 +90,8 @@ typedef struct {                         /* type of structure for a database han
   VILLA *mdidx;                          /* database for the modification date */
   VILLA *idx;                            /* temporary index */
   int (*next)(VILLA *);                  /* temporary cursor function */
+  CURIA *filedb;                         /* database for files */
+  DEPOT *fileidx;                        /* database for file attributes */
   char *cdate;                           /* creation date */
   char *mdate;                           /* modification date */
   char *errmsg;                          /* error message */
@@ -216,6 +221,59 @@ int postarticle(RBBS *rbbs, ARTICLE *art, const char *id);
 int deletearticle(RBBS *rbbs, const char *id);
 
 
+/* Get an attribute of a database.
+   `rbbs' specifies a database handle.
+   `name' specifies the name of an attribute.
+   The return value the value of the attribute or `NULL' if error. */
+char *getattribute(RBBS *rbbs, const char *name);
+
+
+/* Set an attribute of a database.
+   `rbbs' specifies a database handle.
+   `name' specifies the name of an attribute.
+   `value' specifies the value of the attribute.  If it is `NULL', the attribute is deleted.
+   The return value is true if successful, else it is false. */
+int setattribute(RBBS *rbbs, const char *name, const char *value);
+
+
+/* Save a file into a database.
+   `rbbs' specifies a database handle.
+   `password' specifies the password of a file.
+   `subject' specifies the subject of the file.
+   `author' specifies the author of the file.
+   `cdate' specifies the creation date of the file.
+   `ptr' specifies the pointer to the content of the file.
+   `size' specifies the size of the content.
+   The return value is true if successful, else it is false. */
+int savefile(RBBS *rbbs, const char *password, const char *subject, const char *author,
+             const char *cdate, const char *ptr, int size);
+
+
+/* Load a file in a database.
+   `rbbs' specifies a database handle.
+   `id' specifies the ID of a file.
+   `sp' specifies the pointer to a variable to which the size of the region of the return
+   value is assigned.  If it is `NULL', no content is loaded.
+   `attrs' specifies a map to contain the attributes, "password", "subject", "author", "cdate",
+   and "size".
+   If it is `NULL' it is not used.
+   The return value is the pointer to the content of the file or `NULL' if it is not found. */
+char *loadfile(RBBS *rbbs, const char *id, int *sp, CBMAP *attrs);
+
+
+/* Save a file into a database.
+   `rbbs' specifies a database handle.
+   `id' specifies the ID of a file.
+   The return value is true if successful, else it is false. */
+int deletefile(RBBS *rbbs, const char *id);
+
+
+/* Get a list of all files in a database.
+   `rbbs' specifies a database handle.
+   The return value is a list handle containing names of all files. */
+CBLIST *listfiles(RBBS *rbbs);
+
+
 /* Make a object of article from an XML string.
    `xml' specifies a string of XML.
    The return value is a handle of an article object. */
@@ -338,23 +396,24 @@ char *arttotext(ARTICLE *art);
 
 /* Serialize an article into a string of HTML.
    `art' specifies a handle of an article object.
+   `systemuri' specifies the URI of the generator of the HTML.
    `bnum' specifies the number of shown blocks in the body.  If it is zero, all responses are
    shown.  If it is negative, the absolute value is calculated and the lower blocks are shown.
    `rnum' specifies the number of shown responses.  If it is zero, all responses are shown.
    If it is negative, the absolute value is calculated and the lower responses are shown.
    The return value is a string of HTML. */
-char *arttohtml(ARTICLE *art, int bnum, int rnum);
+char *arttohtml(ARTICLE *art, const char *systemuri, int bnum, int rnum);
 
 
 /* Serialize an article into a string of Atom.
    `art' specifies a handle of an article object.
-   `prefix' specifies the prefix of the URI of the generator of the alternative HTML content.
+   `systemuri' specifies the URI of the generator of the alternative HTML content.
    `bnum' specifies the number of shown blocks in the body.  If it is zero, all responses are
    shown.  If it is negative, the absolute value is calculated and the lower blocks are shown.
    `rnum' specifies the number of shown responses.  If it is zero, all responses are shown.
    If it is negative, the absolute value is calculated and the lower responses are shown.
    The return value is a string of Atom. */
-char *arttoatom(ARTICLE *art, const char *prefix, int bnum, int rnum);
+char *arttoatom(ARTICLE *art, const char *systemuri, int bnum, int rnum);
 
 
 /* Get the jet lag by minute.
