@@ -1,6 +1,6 @@
 /*************************************************************************************************
  * Administration Utility for RBBS
- *                                                      Copyright (C) 2003-2004 Mikio Hirabayashi
+ *                                                      Copyright (C) 2003-2005 Mikio Hirabayashi
  * This file is part of RBBS, a web-based bulletin board system.
  * RBBS is free software; you can redistribute it and/or modify it under the terms of the GNU
  * General Public License as published by the Free Software Foundation; either version 2 of the
@@ -18,7 +18,6 @@
 
 #define HTMLTYPE    "text/html"          /* content-type of HTML */
 #define ENCODING    "UTF-8"              /* encoding of the page */
-#define LISTQUERY   "action=1"           /* query to get the list */
 
 enum {                                   /* enumeration for the format of output data */
   FTXML,                                 /* original XML */
@@ -30,12 +29,11 @@ enum {                                   /* enumeration for the format of output
 
 /* global variables */
 const char *g_progname = NULL;           /* program name */
-const char *g_rbbsuri = NULL;            /* base URI of the RBBS system */
+const char *g_systemuri = NULL;            /* base URI of the RBBS system */
 const char *g_dumplang = NULL;           /* language of the dumped data */
 const char *g_dumptitle = NULL;          /* title of the dump data */
 const char *g_dumpauthor = NULL;         /* author of the dump data */
 const char *g_dumpsummary = NULL;        /* summary of the dump data */
-const char *g_artprfx = NULL;            /* prefix of articles in HTML format */
 
 
 /* function prototypes */
@@ -96,21 +94,17 @@ int main(int argc, char **argv){
 /* set global variables */
 void setglobal(void){
   const char *tmp;
-  char *buf;
-  g_rbbsuri = "http://localhost/rbbs/rbbs.cgi";
+  g_systemuri = "http://localhost/rbbs/rbbs.cgi";
   g_dumplang = "en";
   g_dumptitle = "RBBS Articles";
   g_dumpauthor = "anonymous";
   g_dumpsummary = "This page includes all articles of the Raving Bulletin Board System.";
-  if((tmp = getenv("RBBSURI")) != NULL) g_rbbsuri = tmp;
+  if((tmp = getenv("RBBSURI")) != NULL) g_systemuri = tmp;
   if((tmp = getenv("RBBSJETLAG")) != NULL) jetlag(atoi(tmp));
   if((tmp = getenv("RBBSDUMPLANG")) != NULL) g_dumplang = tmp;
   if((tmp = getenv("RBBSDUMPTITLE")) != NULL) g_dumptitle = tmp;
   if((tmp = getenv("RBBSDUMPAUTHOR")) != NULL) g_dumpauthor = tmp;
   if((tmp = getenv("RBBSDUMPSUMMARY")) != NULL) g_dumpsummary = tmp;
-  buf = cbsprintf("%s?id=", g_rbbsuri);
-  cbglobalgc(buf, free);
-  g_artprfx = buf;
 }
 
 
@@ -401,12 +395,12 @@ int procexport(const char *name, const char *id, int ftype){
       free(xml);
       break;
     case FTHTML:
-      xml = arttohtml(art, 0, 0);
+      xml = arttohtml(art, g_systemuri, 0, 0);
       printf("%s", xml);
       free(xml);
       break;
     case FTATOM:
-      xml = arttoatom(art, g_artprfx, 0, 0);
+      xml = arttoatom(art, g_systemuri, 0, 0);
       printf("%s", xml);
       free(xml);
       break;
@@ -513,7 +507,7 @@ int procdump(const char *name, int ftype){
               HTMLTYPE, ENCODING);
       xprintf("<meta name=\"author\" content=\"%@\" />\n", g_dumpauthor);
       xprintf("<meta name=\"generator\" content=\"RBBS %@\" />\n", RBBS_VERSION);
-      xprintf("<link rel=\"contents\" href=\"%@?%s\" />\n", g_rbbsuri, LISTQUERY);
+      xprintf("<link rel=\"contents\" href=\"%@?action=2\" />\n", g_systemuri);
       xprintf("</head>\n");
       xprintf("<body>\n");
       xprintf("<h1>%@</h1>\n", g_dumptitle);
@@ -524,7 +518,7 @@ int procdump(const char *name, int ftype){
       xprintf("<?xml version=\"1.0\" encoding=\"%@\"?>\n", ENCODING);
       xprintf("<feed version=\"0.3\" xmlns=\"http://purl.org/atom/ns#\" xml:lang=\"%@\">\n",
               g_dumplang);
-      xprintf("<link rel=\"alternate\" type=\"%@\" href=\"%@\"/>\n", HTMLTYPE, g_rbbsuri);
+      xprintf("<link rel=\"alternate\" type=\"%@\" href=\"%@\"/>\n", HTMLTYPE, g_systemuri);
       xprintf("<title>%@</title>\n", g_dumptitle);
       xprintf("<author>\n");
       xprintf("<name>%@</name>\n", g_dumpauthor);
@@ -533,7 +527,7 @@ int procdump(const char *name, int ftype){
       xprintf("<modified>%@</modified>\n", tmp);
       free(tmp);
       xprintf("<tagline>%@</tagline>\n", g_dumpsummary);
-      xprintf("<generator name=\"RBBS\">%@?version=%?</generator>\n", RBBS_URI, RBBS_VERSION);
+      xprintf("<generator>RBBS %@</generator>\n", RBBS_VERSION);
       break;
     default:
       xprintf("<?xml version=\"1.0\" encoding=\"%@\"?>\n", ENCODING);
@@ -555,13 +549,13 @@ int procdump(const char *name, int ftype){
         free(xml);
         break;
       case FTHTML:
-        xml = arttohtml(art, 0, 0);
+        xml = arttohtml(art, g_systemuri, 0, 0);
         printf("%s", xml);
         free(xml);
         xprintf("<hr />\n");
         break;
       case FTATOM:
-        xml = arttoatom(art, g_artprfx, 0, 0);
+        xml = arttoatom(art, g_systemuri, 0, 0);
         printf("%s", xml);
         free(xml);
         break;
